/* filter.c */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <fftlib.h>
#include "stmgen.h"

double  getdouble();
int     getkey();
int     getsig(char *, float *, int);
long    getlong();
void    beep();
void    butterworth();
void    bwfilt();
void    convolve();
void    differ();
void    integr();
void    inverse();
void    plotspec(char *, int, double, float *, int);
void    ss();
void    wiener();
void    zerophase();

extern char fn[40];
extern float freq;
extern float dur;
extern float delt;
extern float amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short maxamp;
extern short nper;
extern short harm[MAXNTONE];

static void cdiv();

static short mxnp;
static float *bstim;

void
filter()
{
    printf("\n");
    printf("\n");
    printf("FILTER STIMULUS:\n");
    printf("\n");
    printf("B - apply Butterworth filter\n");
    printf("C - convolve stimulus with a filter response\n");
    printf("I - construct an inverse filter response\n");
    printf("W - construct a Wiener filter from a replication\n");
    printf("N - Integrate\n");
    printf("D - Differentiate\n");
    printf("Z - apply zero-phase filter (12 dB/oct)\n");
    printf("? ");
    switch (getkey()) {
    case 'B':
	butterworth();
	break;
    case 'C':
	convolve();
	break;
    case 'I':
	inverse();
	break;
    case 'W':
	wiener();
	break;
    case 'N':
	integr();
	break;
    case 'D':
	differ();
	break;
    case 'Z':
	zerophase();
	break;
    }
}

void
inverse()
{
    short     i, j, k, nfft, nfrq, n1, n2, nn;
    double f1 = 0.0, t1 = 0.0, f2, t2, nyfr;

    mxnp = MAXNPER / 2;
    if (nper > mxnp) {
        printf("\n\ntoo many points in stimulus\n");
        beep();
        return;
    }
    bstim = astim + mxnp;
    printf("\n\nInverse filter assumes that the stimulus file has been read.\n");
    if (getsig("System response file", bstim, mxnp) <= 0)
        return;
    plotspec(fn, nper, delt, bstim, 0);
    nyfr = (delt > 0.0) ? 0.5 / delt : 1.0;
    f2 = nyfr;
    t2 = delt * nper;
    printf("\n");
    f1 = getdouble("lowest frequency (Hz) = ", f1);
    f2 = getdouble("highest frequency (Hz) = ", f2);
    t1 = getdouble("start time (sec) = ", t1);
    t2 = getdouble("stop time (sec) = ", t2);
    printf("\n\nComputing inverse filter ... ");
    for (nfft = 1; nfft < nper && nfft * 2 < mxnp; nfft *= 2)
        continue;
    nfrq = nfft / 2 + 1;
    n1 = (short) (f1 * nfrq / nyfr + 0.5);
    n2 = (short) (f2 * nfrq / nyfr + 0.5);
    printf(" %d,%d", n1, n2);
    fast(astim, nfft);
    fast(bstim, nfft);
    /* 
     * divide the simulus spectrum by the response spectrum
     * in the specified frequency band
     */
    for (i = n1; i < n2; i++) {
        j = i * 2;
        cdiv(astim + j, bstim + j, astim + j);
    }
    /* fix-up spectrum outside of this band */
    astim[0] = 0.0;
    astim[1] = 0.0;
    k = n1 * 2;
    for (i = 1; i < n1; i++) {
        j = i * 2;
        astim[j] = astim[k];
        astim[j + 1] = astim[k + 1];
    }
    k = (n2 - 1) * 2;
    for (i = n2; i < (nfrq - 1); i++) {
        j = i * 2;
        astim[j] = astim[k];
        astim[j + 1] = astim[k + 1];
    }
    k = (nfrq - 1) * 2;
    astim[k] = 0.0;
    astim[k + 1] = 0.0;
    fsst(astim, nfft);
    ss();
    nper = nfft;
    n1 = (short) (t1 / delt);
    n2 = (short) (t2 / delt);
    nn = n2 - n1 + 1;
    if (nn > nper)
        nn = nper;
    while (n1 < 0)
        n1 += nper;
    n2 = (nn + n1 - 1) % nper;
    printf(";  %d,%d", n1, n2);
    for (i = nn; i < nper; i++)
        astim[i] = 0;
}

void
wiener()
{
    short     i, j, nfft, nfrq, n1, n2, wf = 0;
    double    ar, ai, br, bi;
    double    abmg, bbmg, ab;
    double    nyfr, f2;

    mxnp = MAXNPER / 2;
    if (nper > mxnp) {
        printf("\n\ntoo many points in initial waveform\n");
        beep();
        return;
    }
    bstim = astim + mxnp;
    printf("\n\nWiener filter assumes that the template file has been read.\n");
    if (getsig("System response file", bstim, mxnp) <= 0)
        return;
    f2 = nyfr = (delt > 0.0) ? 0.5 / delt : 1.0;
    f2 = getdouble("highest frequency (Hz) = ", f2);
    wf = getlong("whitening filter (0=no, 1=yes) = ", (long) wf);
    for (nfft = 1; nfft < nper && nfft * 2 < mxnp; nfft *= 2)
        continue;
    fast(astim, nfft);
    fast(bstim, nfft);
    nfrq = nfft / 2 + 1;
    n1 = 1;
    n2 = (short) (f2 * nfrq / nyfr + 0.5);
    for (i = n1; i < n2; i++) {
        j = i * 2;
	ar = astim[j];
	ai = astim[j + 1];
	br = bstim[j];
	bi = bstim[j + 1];
	ab = ar * br + ai * bi;
	abmg = (ab > 0) ? sqrt(ab) : 0;
	bbmg = sqrt(br * br + bi * bi);
	astim[j] = (bbmg > 0) ? abmg / bbmg : abmg * 1e30;
	astim[j + 1] = 0;
    }
    astim[0] = astim[1] = 0.0;
    for (i = n2 * 2; i < (nfft + 2); i++)
	astim[i] = 0.0;
    fsst(astim, nfft);
    ss();
}

void
convolve()
{
    short     i, j, nfft, nfrq;
    float ar, ai, br, bi;

    mxnp = MAXNPER / 2;
    if (nper > mxnp) {
        printf("\n\ntoo many points in stimulus\n");
        beep();
        return;
    }
    bstim = astim + mxnp;
    printf("\n\nConvolution assumes that the stimulus file has been read.\n");
    if (getsig("Filter response file", bstim, mxnp) <= 0)
        return;
    printf("\n\nComputing convolution with filter response ... ");
    for (nfft = 1; nfft < nper && nfft * 2 < mxnp; nfft *= 2)
        continue;
    nfrq = nfft / 2 + 1;
    fast(astim, nfft);
    fast(bstim, nfft);
    for (i = 0; i < nfrq; i++) {
        j = i * 2;
        ar = astim[j];
        ai = astim[j + 1];
        br = bstim[j];
        bi = bstim[j + 1];
        astim[j] = ar * br - ai * bi;
        astim[j + 1] = ai * br + ar * bi;
    }
    fsst(astim, nfft);
    ss();
}

void
cdiv(a, b, c)
float *a, *b, *c;
{
    double bmagsq, cr, ci;

    bmagsq = b[0] * b[0] + b[1] * b[1];
    if (bmagsq < 1e-9)
        bmagsq = 1e-9;
    cr = (a[0] * b[0] + a[1] * b[1]) / bmagsq;
    ci = (a[1] * b[0] - a[0] * b[1]) / bmagsq;
    c[0] = cr;
    c[1] = ci;
}


double
mxas()
{
    short   i;
    double  ampmax;

    ampmax = 1e-30;
    for (i = 0; i <= nper; i++)
	if (ampmax < astim[i])
	    ampmax = astim[i];
	else if (ampmax < -astim[i])
	    ampmax = -astim[i];
    return(ampmax);
}

void
butterworth()
{
    double     a1, a2;
    static int npole = 0;
    static double f1 = 0, f2 = 0;

    printf("\n");
    npole = getlong("number of poles = ", (long) npole);
    f1 = getdouble("lowest frequency (high-pass filter) (Hz) = ", f1);
    f2 = getdouble("highest frequency (low-pass filter) (Hz) = ", f2);
    if ((nper <= 0) || (delt <= 0))
	return;
    a1 = mxas();
    bwfilt(astim, nper, f1 * delt, npole, 1);	/* high-pass filter */
    bwfilt(astim, nper, f2 * delt, npole, 0);	/* low-pass filter */
    a2 = mxas();
    printf("filter gain = %6.4f (%f/%f)\n", a2 / a1, a2, a1);
    ss();
}

/* bwfilt - Butterworth low-pass/high-pass filter (bilinear transform IIR)
 *
 *      ht    - pointer to float array
 *      npts  - number of points in array
 *      r     - ratio of cut-off frequency to sample frequency
 *      npole - number of complex poles (must be even)
 *      hpf   - high-pass flag (0=low-pass, 1=high-pass)
 */

void
bwfilt(ht, npts, r, npole, hpf)
float *ht;
int npts, npole, hpf;
double r;
{
    int i, j;
    double aa, bb, rr;
    double a0, a1, a2;
    double b0, b1, b2;
    double x0, x1, x2;
    double y0, y1, y2;
    static double pi = 3.14159265;

    if (npole > 1 && r > 0) { 
        rr = tan(pi * r);
        bb = rr * rr;
        for (j = 0; j < npole / 2; j++) {
            aa = 2 * rr * sin((j + 0.5) * pi / npole);
            b0 = 1 + aa + bb;
            b1 = 2 * (bb - 1) / b0;
            b2 = (1 - aa + bb) / b0;
	    if (hpf) {
		a0 = 1 / b0;
		a1 = -2 / b0;
		a2 = 1 / b0;
	    } else {
		a0 = bb / b0;
		a1 = 2 * bb / b0;
		a2 = bb / b0;
	    }
            y0 = y1 = 0;
            x0 = x1 = 0;
            for (i = 0; i < npts; i++) {
                x2 = x1;
                x1 = x0;
                x0 = ht[i];
                y2 = y1;
                y1 = y0;
                y0 = a0 * x0 + a1 * x1 + a2 * x2 - b1 * y1 - b2 * y2;
                ht[i] = y0;
            }
        }
    }
}

void
integr()
{
    int     i;

    for (i = 1; i < nper; i++)
        astim[i] += astim[i - 1];
    ss();
}

void
differ()
{
    int     i;

    for (i = nper - 1; i > 0; i--)
        astim[i] -= astim[i - 1];
    ss();
}

void
zpf(float *x, int n, float f1, float f2, int e1, int e2)
{
    float s1, s2, p1, p2, wt;
    int i, j, k, nf;

    f1 *= n * delt;
    f2 *= n * delt;
    nf = n / 2;
    fast(x, n);
    x[0] = x[n] = 0;
    for (i = 1, j = 2; i < nf; i++) {
        s1 = (f1 > 0) ? f1 / i : 0;
	for(k = 0, p1 = 1; k < e1; k++)
	    p1 *= s1;
        s2 = (f2 > 0) ? i / f2 : 0;
	for(k = 0, p2 = 1; k < e2; k++)
	    p2 *= s2;
        wt = 1 / (1 + p1 + p2);
        x[j++] *= wt;
        x[j++] *= wt;
    }
    fsst(x, n);
}

void
zerophase()
{
    double     a1, a2;
    static int e1 = 2, e2 = 2;
    static double f1 = 0, f2 = 0;

    printf("\n");
    f1 = getdouble("lowest frequency (high-pass filter) (Hz) = ", f1);
    e1 = getlong("exponent = ", (long) e1);
    f2 = getdouble("highest frequency (low-pass filter) (Hz) = ", f2);
    e2 = getlong("exponent = ", (long) e2);
    if ((nper <= 0) || (delt <= 0))
	return;
    a1 = mxas();
    zpf(astim, nper, f1, f2, e1, e2);
    a2 = mxas();
    printf("filter gain = %6.4f (%f/%f)\n", a2 / a1, a2, a1);
    ss();
}
