/*
 * impulse - Compute impulse stimulus
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "stmgen.h"

#define MAXNPOLE    64

double  getdouble();
long    getlong();
void    fsst();
void    ss();
void    zerstm();

extern float freq;
extern float dur;
extern float delt;
extern float amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short maxamp;
extern short nper;
extern short harm[MAXNTONE];

struct cmplx {float x,y;};

static short npole = 2;
static double tpi = (2.0 * 3.1415927);
static struct cmplx pole[MAXNPOLE], frqrsp, a, b, c, t;

void
impulse()
{
    short     i, ip;
    double  fco, dang, angle, dw, w, bms;
    double  eps = 1e-9;

    printf("\n\n");
    freq = getdouble("low-pass cut-off frequency (Hz) = ", freq);
    npole = getlong("number of poles (6 dB/octave/pole) = ", (long) npole);
    if (nper <= 0 || delt <= 0)
	return;
    if (npole <= 0)
        freq = 0.0;
    else if (freq < 50)
	freq *= 1000;
    printf("cutoff = %d dB per octave above %.0f Hz\n", npole * 6, freq);

/*
 * define stimulus parameters
 */
    harm[0] = 0;
    amp[0] = 1.0;
    phase[0] = 0.0;

/*
 * compute stimulus
 */
    zerstm();
    fco = freq * delt;    
    printf("(f_cutoff / f_sample) = %.3f\n", fco);
    if (fco < eps || fco > 0.5) {
        astim[0] = 1;
        return;
    }    
/*
 * compute poles in s-plane
 */
    dang = tpi / (npole * 4.0);
    for(ip = 0; ip < npole; ip++) {
        angle = dang * (2 * ip + 1);
        pole[ip].x = -sin(angle) * fco;
        pole[ip].y = cos(angle) * fco;
    }

/*
 * compute frequency response
 */
    dw = 1.0 / nper;
    for(i = 0; i <= (nper / 2); i++) {
        w = i * dw;
        frqrsp.x = 1.0;
        frqrsp.y = 0.0;
        for(ip = 0; ip < npole; ip++) {

            /* a = pole                 */
            a.x = pole[ip].x;
            a.y = pole[ip].y;

            /* b = conj(pole - iw)      */
            b.x = a.x;
            b.y = w - a.y;
            bms = b.x * b.x + b.y * b.y;
            if (bms < eps)
                bms = eps;

            /* c = pole / (pole - iw)   */
            c.x = (a.x * b.x - a.y * b.y) / bms;
            c.y = (a.y * b.x + a.x * b.y) / bms;

            t.x = frqrsp.x * c.x - frqrsp.y * c.y;
            t.y = frqrsp.y * c.x + frqrsp.x * c.y;
            frqrsp.x = t.x;
            frqrsp.y = t.y;
        }
        astim[2 * i] = frqrsp.x;
        astim[2 * i + 1] = frqrsp.y;
    }

/*
 * do fourier synthesis
 */
    fsst(astim, nper);

/*
 * scale to maxamp
 */
    ss();
}

void
noise()
{
    short i, j;
    double avg, sum = 0.0;
    static short nsum = 1;
    
    printf("\n\n");
    nsum = getlong("number of random values to sum = ", (long) nsum);
    for (i = 0; i < nper; i++) {
	astim[i] = rand();
        for (j = 1; j < nsum; j++)
            astim[i] += rand();
        sum += astim[i];
    }
    avg = sum / nper;
    for (i = 0; i < nper; i++) {
	astim[i] -= avg;
    }
    ss();
}

void
delta()
{
    int i;
    
    for (i = 1; i < nper; i++)
	astim[i] = 0;
    astim[0] = maxamp;
}

