/* kb.c */
 
#include <stdio.h>
#include <termio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/time.h>

#define TRUE    1
#define FALSE   0
/*
 * keyboard stuff 
 */
static struct termio kbd_setup;
static struct termio kbd_reset;
static int    kbd_initted = FALSE;
static int    kbd_isatty;
static int    kbd_lastchr;
static int    kbd_filedsc;
static enum { normal, test, wait, unknown } kbd_mode = unknown;

void
kbd_restore(void)
{
    if(kbd_initted && kbd_isatty && (kbd_mode != normal)) {
	ioctl(kbd_filedsc,TCSETA,&kbd_reset);
	kbd_mode = normal;
    }
}

void
kbd_init(void)
{
    if(!kbd_initted) {
	kbd_initted = TRUE;
	kbd_lastchr = EOF;
	kbd_filedsc = fileno(stdin);
	kbd_isatty  = isatty(kbd_filedsc);	
	if(kbd_isatty) {
	    ioctl(kbd_filedsc,TCGETA,&kbd_setup);
	    ioctl(kbd_filedsc,TCGETA,&kbd_reset);
	    kbd_setup.c_lflag &= ~(ICANON | ECHO );
	    kbd_setup.c_iflag &= ~(INLCR  | ICRNL);
	    atexit(kbd_restore);
	    kbd_mode = normal;
	}
    }
}

int
_kbhit(void)
{
	unsigned char buf[4];
	if(!kbd_initted) {
	    kbd_init();
	}
	if(!kbd_isatty) {
	    return(TRUE);
	}
	if(kbd_lastchr != EOF) {
	    return(TRUE);
	}
	if(kbd_mode != test) {
	    kbd_setup.c_cc[VMIN]  = 0;
	    kbd_setup.c_cc[VTIME] = 0;
	    if(ioctl(kbd_filedsc,TCSETAW,&kbd_setup) == EOF) return(FALSE);
	    kbd_mode = test;
	}
	if(read(kbd_filedsc,buf,1) > 0) {
	    kbd_lastchr = buf[0];
	    return(TRUE);
	}
	return(FALSE);
}

int 
_getch(void)
{
	unsigned char buf[4];
	if(!kbd_initted) {
	    kbd_init();
	}
	if(!kbd_isatty) {
	    return(getc(stdin));
	}
	if(kbd_lastchr != EOF) {
	    buf[0] = kbd_lastchr;
	    kbd_lastchr = EOF;
	    return(buf[0]);
	}
	if(kbd_mode != wait) {
	    kbd_setup.c_cc[VMIN]  = 1;
	    kbd_setup.c_cc[VTIME] = 0;
	    if(ioctl(kbd_filedsc,TCSETAW,&kbd_setup) == EOF) return(EOF);
	    kbd_mode = wait;
	}
	if(read(kbd_filedsc,buf,1) > 0) {
	    return(buf[0]);
	}
	return(EOF);
}

