/* plotstm.c */

#include <stdio.h>
#define ANSI_C
#include <output.h>

#define BELL     7

extern short pc_video_adapt;

double  sqrt();
unsigned strlen();
void ytick();
void gwindo();
void dwindo();
void box();
void g_init();
void g_close();
void tpln();
void xtick();

static void pltfy();

static float dx1 = 0., dy1 = 0., dx2 = 10.5, dy2 = 8.;
static float gx1 = 0., gy1 = 0., gx2 = 10.5, gy2 = 8.;
static float xg = 1., yg = 1., xo = 0., yo = 0.;

void
plotstm(char *ilstim, int npts, double delt, float *outbuf, int maxamp)
{
/*
 * plot the waveform
 */
    char    str[80];
    short   i;
    double  x1, y1, x2, y2, y, ymn, ymx, ydc, sum, rms;
    double   a;

    if (npts <= 0)
	return;
    a = (maxamp < 1) ? 1 : 1.1 * maxamp;
    ymn = ymx = outbuf[0];
    sum = 0.0;
    for (i = 0; i < npts; i++) {
	y = outbuf[i];
	sum += y;
	if (ymx < y)
	    ymx = y;
	if (ymn > y)
	    ymn = y;
    }
    ydc = sum / npts;
    for (i = 0; i < npts; i++) {
	y = outbuf[i] - ydc;
	sum += y * y;
    }
    rms = sqrt(sum / npts);
    g_init();
    sprintf(str, "file = %s", ilstim);
    text(0.5, 0.8, str);
    sprintf(str, "amplitude = %d", maxamp);
    text(6.5, 0.8, str);
    sprintf(str, "npts = %d", npts);
    text(0.5, 0.4, str);
    if (npts * delt > 1.0)
	sprintf(str, "duration = %.3f sec", npts * delt);
    else
	sprintf(str, "duration = %.3f msec", npts * delt * 1000);
    text(2.5, 0.4, str);
    if (delt > 0.0) {
	if (delt > 0.001)
	    sprintf(str, "rate = %.1f Hz", 1.0 / delt);
	else
	    sprintf(str, "rate = %.1f kHz", 0.001 / delt);
	text(6.5, 0.4, str);
    }
    sprintf(str, "min = %.0f    max = %.0f    dc = %.3f    rms = %.3f",
	ymn, ymx, ydc, rms);
    text(0.5, 0.2, str);
    x1 = 0;
    x2 = 1;
    y1 = -a;
    y2 = a;
    dwindo(x1, y1, x2, y2);	/* define the data window */
    x1 = 0.25;
    x2 = 10.25;
    y1 = 1.75;
    y2 = 7.75;
    gwindo(x1, y1, x2, y2);	/* define the 1st graphics window */
    box(x1, y1, x2, y2);	/* outline the plotting area */
    ytick(x1, x2, (x2 - x1), 0.0);
    pltfy(npts, outbuf);	/* plot entire stimulus */
    g_close();
}

void
g_init()
{
    (void) opnout(defdev());
}

void
g_close()
{
    clsout();
}

/* pltfy - plot float array, y values only */

void
pltfy(int n, float *fy)
{
    short   i;
    float   x[2], y[2], dx;

    dx = 1.0 / n;
    for (i = 0; i < n; i++) {
	x[1] = i * dx;
	y[1] = fy[i];
	if (i > 0)
	    tpln(x, y);
	x[0] = x[1];
	y[0] = y[1];
    }
}

void
ytick(x1, x2, tl, yv)
double  x1, x2, tl, yv;
{
    float   c[2], r[2];

    r[0] = yv * yg + yo;
    r[1] = r[0];
    c[0] = x1;
    c[1] = x1 + tl;
    line(2, c, r);
    c[0] = x2;
    c[1] = x2 - tl;
    line(2, c, r);
}

void
gwindo(x1, y1, x2, y2)
float   x1, y1, x2, y2;
{
    gx1 = x1;
    gy1 = y1;
    gx2 = x2;
    gy2 = y2;
    xg = (gx2 - gx1) / (dx2 - dx1);
    yg = (gy2 - gy1) / (dy2 - dy1);
    xo = gx1 - dx1 * xg;
    yo = gy1 - dy1 * yg;
}

void
dwindo(x1, y1, x2, y2)
float   x1, y1, x2, y2;
{
    dx1 = x1;
    dy1 = y1;
    dx2 = x2;
    dy2 = y2;
    xg = (gx2 - gx1) / (dx2 - dx1);
    yg = (gy2 - gy1) / (dy2 - dy1);
    xo = gx1 - dx1 * xg;
    yo = gy1 - dy1 * yg;
}

void
box(x1, y1, x2, y2)
float   x1, y1, x2, y2;
{
    float   c[5], r[5];

    c[0] = c[3] = c[4] = x1;
    r[0] = r[1] = r[4] = y1;
    c[1] = c[2] = x2;
    r[2] = r[3] = y2;
    line(5, c, r);
}

void
tpln(x, y)
float  *x, *y;
{
    float   xx[2], yy[2];

    xx[0] = xo + xg * x[0];
    yy[0] = yo + yg * y[0];
    xx[1] = xo + xg * x[1];
    yy[1] = yo + yg * y[1];
    line(2, xx, yy);
}

void
xtick(y1, y2, tl, xv)
double  y1, y2, tl, xv;
{
    float   c[2], r[2];

    c[0] = xv * xg + xo;
    c[1] = c[0];
    r[0] = y1;
    r[1] = y1 + tl;
    line(2, c, r);
    r[0] = y2;
    r[1] = y2 - tl;
    line(2, c, r);
}

