/* getstm - get stimulus from disk */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "stmgen.h"
#ifdef MSDOS
#include <io.h>
#else
#define _O_BINARY    0
#define _O_RDONLY    O_RDONLY
#define _close       close
#define _read        read
#endif
#ifdef linux
#include <unistd.h>
void	swab();
#endif
#define _open        open
#define _swab        swab
#define _lseek       lseek

#define FT_ILS      0
#define FT_MAT      1
#define wswapb(a,b) _swab((char *)(a),(char *)(a),(int)((b)*sizeof(short)))

char   *strchr();
int     get_stim(float *, int);
int     getsig(char *, float *, int);
int     opnblk();
int     _read();
int     rdblk();
int     rdils();
int     rdilo();
int     rdapl();
int     rdmat();
int     guess_file_type(char *);
char   *tfn;
unsigned strlen();
void    clsblk();
void    getstr();

extern char fn[40];
extern float delt, f0, amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short cpu_lsb;
extern short nper, nblk, maxamp, harm[MAXNTONE];
extern short params[256];
extern short istim[MAXNPER];
extern short filfmt;

static double rate = 10000;

void
lswapb(long *a, int n)
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) a++;
	tmp = ch[3];
	ch[3] = ch[0];
	ch[0] = tmp;
	tmp = ch[2];
	ch[2] = ch[1];
	ch[1] = tmp;
    }
}

void
dswapb(double *a, int n)
{
    unsigned char *ch, tmp;

    while (n-- > 0) {
	ch = (unsigned char *) a++;
	tmp = ch[7];
	ch[7] = ch[0];
	ch[0] = tmp;
	tmp = ch[6];
	ch[6] = ch[1];
	ch[1] = tmp;
	tmp = ch[5];
	ch[5] = ch[2];
	ch[2] = tmp;
	tmp = ch[4];
	ch[4] = ch[3];
	ch[3] = tmp;
    }
}

int
getstm()
{
    return (getsig("Read stimulus from file", astim, MAXNPER));
}

int
getsig(char *prompt, float *stim, int size)
{
    printf("\n%s: ", prompt);
    getstr(fn);
    if (strlen(fn) <= 0)
	return (0);

    return (get_stim(stim, size));
}

int
get_stim(float *stim, int size)
{
    int i, ff, status = -1;
    	
    ff = guess_file_type(fn);
    if (ff >= 0)
        filfmt = ff;

    switch (filfmt) {
    case FT_ILS:
        status = rdils(stim, size);
        break;
    case FT_MAT:
        status = rdmat(stim, size);
        break;
    }

    if (status < 0) {
	(void) fprintf(stderr, "can't open '%s'\n", fn);
    } else {
        delt = (rate > 0.0) ? (1.0 / rate) : 0.0;
        (void) fprintf(stderr, "%s: nper=%d rate=%.0f\n", fn, nper, rate);
        for (i = nper; i < size; i++)
            stim[i] = 0;
    }

    return (status);
}

#define NBLKS   ils_hdr[5]	/* number of disk blocks */
#define SF      ils_hdr[61]	/* sampling frequency */
#define TYPE    ils_hdr[62]	/* type of file */

int
guess_file_type(char *ifn)
{
    short  *ils_hdr;
    long   *mat_hdr;
    int     n, file;
    unsigned int mode;
    static int hdrsiz = 256;
    static int sdf = -32000;

    mode = _O_RDONLY | _O_BINARY;
    if ((file = _open(ifn, mode)) == -1)
	return (-1);
    n = _read(file, (char *) istim, hdrsiz);
    (void) _lseek(file, 0L, 0);
    if (n <= 0)
	return (-1);

    ils_hdr = (short *) istim;	/* ILS */
    if (TYPE == sdf && NBLKS > 0 && SF > 0)
	return (FT_ILS);

    mat_hdr = (long *) istim;	/* MATLAB ? */
    if (mat_hdr[0] < 0 || mat_hdr[0] > 9999)
	lswapb(mat_hdr, 5);
    if (mat_hdr[0] >= 0 && mat_hdr[0] <= 9999)
	if (mat_hdr[1] > 0 && mat_hdr[2] > 0)
	    if (mat_hdr[4] > 0)
		    return (FT_MAT);

    return (-1);
}

int
rdils(float *stim, int size)
{
    short fd, blocks, *ilshdr;
    long i;

    fd = opnblk(fn);	/* open the file */
    if (fd < 0)
	return (fd);

    ilshdr = (short *) &params[0];
    (void) rdblk(fd, 0, ilshdr, 1);
    if (ilshdr[62] != ILS) {
	_close(fd);
	(void) fprintf(stderr, "'%s' is not an ILS 'sampled data' file\n", fn);
	return (ILS);
    }
    nper = ilshdr[5] * BLKSIZ;
    rate = ilshdr[61] * pow(10.0, (double) ilshdr[60]);
    blocks = size / BLKSIZ;
    if (ilshdr[5] < blocks)
	blocks = ilshdr[5];

    nper = rdblk(fd, 1, istim, blocks);	/* read data */
    _close(fd);

    for (i = 0; i < nper; i++)
	stim[i] = istim[i];

    return (fd);
}

static double
rd_mat_val(int fd, int dtyp, int bswp)
{
    double dtmp;
    float  ftmp;
    long   ltmp;
    short  stmp;
    unsigned long utmp;
    
    if (dtyp == 0) {
        (void) _read(fd, &dtmp, 8);
        if (bswp) dswapb(&dtmp, 1);
    } else if (dtyp == 1) {
        (void) _read(fd, &ftmp, 4);
        if (bswp) lswapb((long *) &ftmp, 1);
        dtmp = ftmp;
    } else if (dtyp == 2) {
        (void) _read(fd, &ltmp, 4);
        if (bswp) lswapb(&ltmp, 1);
        dtmp = ltmp;
    } else if (dtyp == 3) {
        (void) _read(fd, &stmp, 2);
        if (bswp) wswapb((short *) &stmp, 1);
        dtmp = stmp;
    } else if (dtyp == 4) {
        (void) _read(fd, &utmp, 4);
        if (bswp) lswapb((long *) &utmp, 1);
        dtmp = utmp;
    }
    return (dtmp);
}

/*
 *  matlab header -
 *	type = MOPT where:
 *	M=0 for pc; 1 for sun
 *	O=0 col or 1 row (ignored);
 *	P= [0,1,2,3,4]=>[r*8,r*4,I*4,I*2,I*4 unsigned integers]
 *	T= 0 for matrix or 1 for text (stored as r*4 numbers 0<i<255).
 */
static void
decode_mopt(long mopt, char *type)
{
    if (mopt < 0 || mopt > 9999)
	lswapb(&mopt, 1);
    type[0] = (mopt / 1000) % 10;
    type[1] = (mopt / 100) % 10;
    type[2] = (mopt / 10) % 10;
    type[3] = mopt % 10;
}

int
rdmat(float *stim, int size)
{
    char    nam[40], type[4];
    double *dstim;
    float   mx;
    int     i, fd, mode, status, bsiz, dtyp, bswp;
    long    hdr[5], ofst, nsmp, *lstim, mxbsz;
    short  *sstim;
    static int bsz[5] = {8, 4, 4, 2, 4};
    unsigned int len;
    static char *dtn[5] = {
	"double", "float", "long", "short", "ulong"
    };

    mode = (int) (_O_RDONLY | _O_BINARY);
    fd = _open(fn, mode, 0);
    if (fd < 0)
	return (fd);

    rate = 50000;
    nper = 0;
    ofst = 0;
    for (;;) {
	nsmp = 0;
	if (_lseek(fd, ofst, 0) < 0)
	    break;
	len = 20;
	status = _read(fd, hdr, len);
	if (status != len)
	    break;
	decode_mopt(hdr[0], type);
        bswp = (type[0] != cpu_lsb);
        dtyp = type[2];
	bsiz = bsz[dtyp];
	if (bswp)
	    lswapb(hdr, 5);
	nsmp = hdr[1] * hdr[2];
	if (nsmp <= 0)
	    break;
	ofst += 20 + hdr[4];
        len = (unsigned int) hdr[4];
        status = _read(fd, nam, len);
	printf("%8ld (%.6s):  %s%s\n", nsmp, dtn[dtyp], nam, hdr[3] ? "*" : "");
	if (nsmp > 1 && type[3] == 0 && hdr[3] == 0)
	    break;
        if (strcmp(nam, "fmax") == 0) {
            rate = rd_mat_val(fd, dtyp, bswp) * 2;
        } else if (strcmp(nam, "rate") == 0) {
            rate = rd_mat_val(fd, dtyp, bswp);
        }
	ofst += nsmp * bsiz;
	if (hdr[3])                 /* complex data */
            ofst += nsmp * bsiz;
    }
    if (nsmp <= 0) {
	_close(fd);
	return (0);
    }
    mxbsz = size * 4L;
    nper = ((nsmp * bsiz) > mxbsz) ? (mxbsz / bsiz) : nsmp;
    status = _read(fd, (char *) stim, nper * bsiz);   
    if (dtyp == 0) {
        dstim = (double *) stim;
        if (bswp) dswapb(dstim, nper);
        for(i = 0; i < nper; i++)
            stim[i] = dstim[i];
    } else if (dtyp == 1) {
        if (bswp) lswapb((long *) stim, nper);
    } else if (dtyp == 2) {
        lstim = (long *) stim;
        if (bswp) lswapb((long *) lstim, nper);
        for(i = 0; i < nper; i++)
            stim[i] = lstim[i];
    } else if (dtyp == 3) {
        sstim = (short *) stim;
        if (bswp) wswapb((long *) sstim, nper);
        for(i = 0; i < nper; i++)
            stim[i] = sstim[i];
    } else {
    	nper = 0;
    }
    _close(fd);

    mx = 1e-20;
    for (i = 0; i < nper; i++) {
        if (mx < stim[i])
            mx = stim[i];
        if (mx < -stim[i])
            mx = -stim[i];
    }
    maxamp = 32000;
    for (i = 0; i < nper; i++)
        stim[i] *= maxamp / mx;

    return (fd);
}

/*****************************************************************************/

int
opnblk(file)
/*
 * open a file in raw mode for block io.
 * return a file descriptor.
 */
char   *file;
{
    int     fd, oflg;

    oflg = (int) (_O_RDONLY | _O_BINARY);
    fd = _open(file, oflg, 0);
    return (fd);
}

int
rdblk(fd, block, buf, blocks)
/* read from fd, blocks into buf starting at block */
short int fd, block, blocks;
short int *buf;
{
    int     nw;
    long    offset;
    unsigned int count, rd;

    offset = block * 512L;
    (void) _lseek(fd, offset, 0);	/* seek from block 0 */
    count = 512 * (unsigned int) blocks;
    rd = _read(fd, (char *) buf, count);
    nw = (int) (rd / 2);
    if (cpu_lsb != 0)
	wswapb(buf, nw);	/* swap bytes */

    return (nw);
}

void
clsblk(fd)
short int fd;
{
    _close(fd);
}
