/* plotspec.c  - plot the spectrum */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fftlib.h>
#include "stmgen.h"

#define BELL     7
#define limit(min,val,max)	((val<min)?min:((val>max)?max:val))
#define _hypot   hypot

double  cdb();
double  cph();
unsigned strlen();
void    box();
void    dwindo();
void    g_close();
void    g_init();
void    gwindo();
void    text();
void    tpln();
void    xtick();
void    ytick();

static void linxtick();
static void logxtick();
static void pltcy();

static double tpi = 6.2831853;

void
plotspec(char *ilstim, int npts, double delt, float *abuf, int logx)
{
    char    str[80];
    short     i, j, nfft, nfrq;
    double  ymn, ymx, mxmg, phrf, mngd, mxgd;
    double  tl = 0.1, dt;
    double x1, y1, x2, y2;
    float   mg, ph, gd, * bbuf;

    if (npts <= 0)
	return;
    printf("\n\nComputing spectrum ... ");
    for (nfft = 1; nfft < npts && nfft * 2 < MAXNPER; nfft *= 2)
        continue;
    nfrq = nfft / 2 + 1;
    bbuf = (float *) calloc(nfft + 2, sizeof(float));
    if (bbuf == NULL)
	return;
    for (i = 0; i < npts; i++)
        bbuf[i] = abuf[i];
    for (i = npts; i < nfft; i++)
        bbuf[i] = 0.0;
    fast(bbuf, nfft);
    mxmg = 0.0;
    phrf = 0.0;
    dt = delt * nfft * 1000;
    mxgd = 0.0;
    mngd = -1.0;
    for (i = 0; i < nfrq; i++) {
        j = i * 2;
        mg = cdb(bbuf[j], bbuf[j + 1]);
        ph = cph(bbuf[j], bbuf[j + 1], phrf);
        gd = (i > 1) ? (ph - phrf) * dt : 0;
        if (mg > mxmg && i > 0)
            mxmg = mg;
        if (gd > mxgd && i > 0)
            mxgd = ceil(gd);
        bbuf[j] = mg;
        bbuf[j + 1] = gd;
        phrf = ph;
    }
    g_init();
    sprintf(str, "Spectrum: file = %s", ilstim);
    text(0.5, 0.1, str);
    dt = 2 * 1000 * delt;
    ymx = mxmg + 3.0;
    ymn = ymx - 100.0;
    dwindo(0.0, ymn, 1.0, ymx);     /* define the data window */
    x1 = 0.5;
    x2 = 10.25;
    y1 = 4.25;
    y2 = 7.75;
    gwindo(x1, y1, x2, y2);     /* define the 1st graphics window */
    box(x1, y1, x2, y2);        /* outline the plotting area */
    if (logx == 0)
        linxtick(y1, y2, dt, tl);
    else
        logxtick(y1, y2, dt, tl, nfrq);
    for (i = 1; i <= 9; i++)
        ytick(x1, x2, tl, ymx - 10.0 * i);
    for (i = 0; i < nfrq; i++) {
        j = i * 2;
	bbuf[j] = limit(ymn, bbuf[j], ymx);
    }
    pltcy(nfrq, bbuf, logx);              /* plot log magnitude */

    dwindo(0.0, mngd, 1.0, mxgd);     /* define the data window */
    x1 = 0.5;
    x2 = 10.25;
    y1 = 0.50;
    y2 = 4.00;
    gwindo(x1, y1, x2, y2);     /* define the 2nd graphics window */
    box(x1, y1, x2, y2);        /* outline the plotting area */
    if (logx == 0)
        linxtick(y1, y2, dt, tl);
    else
        logxtick(y1, y2, dt, tl, nfrq);
    for (i = (short) mngd; i <= mxgd; i++)
        ytick(x1, x2, tl, (double) i);
    ytick(x1, x2, (x2 - x1), 0.0);
    for (i = 0; i < nfrq; i++) {
        j = i * 2 + 1;
	bbuf[j] = limit(mngd, bbuf[j], mxgd);
    }
    pltcy(nfrq, bbuf + 1, logx);              /* plot group delay */
    g_close();

    free(bbuf);
}

/* pltcy - plot complex array */
void
pltcy(int n, float *cy, int logx)
{
    short     i, j;
    float   x[2], y[2], dx;

    dx = (logx == 0) ? 1.0 / n : 1.0 / log10((double) n);
    x[0] = 0;
    y[0] = cy[0];
    for (i = 1; i < n; i++) {
        j = i * 2;
	x[1] = (logx == 0) ? i * dx : log10((double) i) * dx;
	y[1] = cy[j];
	if (i > 0)
	    tpln(x, y);
	x[0] = x[1];
	y[0] = y[1];
    }
}

double
cdb(x, y)
double x, y;
{
    double mg, db;

    mg = _hypot(x, y);
    db = (mg < 1.e-39) ? -800.0 : 20.0 * log10(mg);
    return (db);
}

double
cph(x, y, r)
double x, y, r;
{
    double ph;

    if (x == 0.0 && y == 0.0)
        return (0.0);
    ph = atan2(x, y) / tpi;
    while (ph - r > 0.5)
        ph -= 1.0;
    while (ph - r < -0.5)
        ph += 1.0;
    while (ph - r < 0)
        ph += 1.0;
    return (ph);
}

void
linxtick(y1, y2, dt, tl)
double y1, y2, tl, dt;
{
    double f, nyfr, dx;
    
    nyfr = (dt > 0.0) ? 1 / dt : 0.001;
    dx = dt;
    for (f = 1; f <= nyfr; f++)
        xtick(y1, y2, tl, f * dx);
}

void
logxtick(y1, y2, dt, tl, n)
double y1, y2, tl, dt;
{
    double f, nyfr, dx, d, nof;

    nyfr = (dt > 0.0) ? 1 / dt : 0.001;
    nof = n / nyfr;
    dx = 1.0 / log10((double) n);
    for (d = 1; d <= nyfr; d *= 10) {
        xtick(y1, y2, tl, log10(d * nof) * dx);
        for (f = d / 10; f < d; f += d / 10)
            xtick(y1, y2, tl / 2, log10(f * nof) * dx);
    }
}
