/* pltstm.c */

#include <stdio.h>

#define BUF_LEN 10240
#define STR_LEN  80

#ifdef linux
void    kbd_init();
void    kbd_restore();
#endif /* linux */

int     getstim(char *, short *, long, float *, int *);
void    plotspec(char *, int, double, float *, int);
void    plotstm(char *, int, double, float *, int);
int     rdils();

long    buflen = BUF_LEN;
short   ibuf[BUF_LEN] = {0};
float   fbuf[BUF_LEN] = {0};
short   cpu_lsb = 0;

static void setlsb();
static void usage();

int
main(int argc, char **argv)
{

    char   *ilstim;
    int     npts, maxamp, i;
    float   rate;
    char    plot_type = 'T';

    while (argc > 1 && argv[1][0] == '-') {
        switch (argv[1][1]) {
        case 'f':
            plot_type = 'F';
            break;
        case 'l':
            plot_type = 'L';
            break;
        default:
            plot_type = 'T';
        }
        argc--;
        argv++;
    }
    if (argc < 2)
        usage();
    ilstim = argv[1];
    setlsb();

/* read ils stimulus */

    maxamp = getstim(ilstim, ibuf, buflen, &rate, &npts);
    for (i = 0; i < npts; i++)
    	fbuf[i] = ibuf[i];

/* plot stimulus to screen */

    switch (plot_type) {
    case 'F':
        plotspec(ilstim, npts, 0.001 / rate, fbuf, 0);
        break;
    case 'L':
        plotspec(ilstim, npts, 0.001 / rate, fbuf, 1);
        break;
    default:
        plotstm(ilstim, npts, 0.001 / rate, fbuf, maxamp);
    }
    return (0);
}

/*
 * getstim - Read ils stimulus file
 */

int
getstim(char *ilstim, short *outbuf, long buflen, float *prate, int *pnpts)
{
    int     i, mx;

    if (rdils(ilstim, outbuf, buflen, prate, pnpts) <= 0) {
	printf("can't open '%s'", ilstim);
	exit(1);
    }
    mx = 0;
    for (i = 0; i < *pnpts; i++)
	if (outbuf[i] > 0 && mx < outbuf[i])
	    mx = outbuf[i];
	else if (outbuf[i] < 0 && mx < -outbuf[i])
	    mx = -outbuf[i];
    return (mx);
}

void
setlsb()
{
    short a = 1;
    char *b = (char *) (&a);

    cpu_lsb = b[1] ? 1 : 0;
}

void
getstr(char *s)
{
#ifdef linux
    fflush(stdout);
    kbd_restore();
    (void) fgets(s, STR_LEN, stdin);
    kbd_init();
#else
    (void) fgets(s, STR_LEN, stdin);
#endif
    s[strlen(s) - 1] = '\0';
}

void
usage()
{
    fprintf(stderr, "usage: pltstm [-option] stim_file\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "option:\n");
    fprintf(stderr, "           -f  frequency spectrum\n");
    fprintf(stderr, "           -l  log-frequency spectrum\n");
    exit(0);
}
