/*
 * stmgen - Compute stimulus files
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "stmgen.h"
#include "version.h"
#ifdef MSDOS
#include <conio.h>
#define _getch getch
#endif
#ifdef linux
int     _getch();
void    kbd_init();
void    kbd_restore();
#endif /* linux */

char    fn[40] = "tmp.stm";
short   cpu_lsb = 0;
short   nblk, harm[MAXNTONE];
short   nper = 1024;
short   ntone = 0;
short   maxamp = 32000;
short   filfmt = 0;
short   params[256];
float   delt = 2e-5;
float   freq = 1000.0;
float   dur = 0.004;
float   f0, amp[MAXNTONE], phase[MAXNTONE];
short   istim[MAXNPER];
float   astim[MAXNPER];

double  getdouble();
int     getstm();
int     getkey();
int     getyes();
long    getlong();
int     get_stim(float *, int);
void    beep();
void    caltone();
void    delta();
void    dispersion();
void    filter();
void    impulse();
void    logsweep();
void    multitone();
void    noise();
void    plotspec(char *, int, double, float *, int);
void    plotstm(char *, int, double, float *, int);
void    strstm();
void    sweep();
void    tobrst();
void    tonegap();
void    winmod();
void    zero();

static void filetype();
static void compute();
static void modify();
static void plot();
static void specval();
static void hfemph();
static void lfemph();
static void rotate();
static void setlsb();

static char *ffstr[2] = {"ILS", "MAT"};

int
main(int ac, char **av)
{
    int     i;

    setlsb();
    if (ac > 1) {
        strcpy(fn, av[1]);
        (void) get_stim(astim, MAXNPER);
    }
    for (;;) {
	printf("\n");
	printf("\n");
	printf("STMGEN version %s (%s mode)\n", VERSION, ffstr[filfmt]);
	printf("\n");
	printf("T - Type of file\n");
	printf("V - Values for rate, length, & amplitude\n");
	printf("C - Compute stimulus\n");
	printf("M - Modify stimulus\n");
	printf("W - Write stimulus to file\n");
	printf("R - Read stimulus from file\n");
	printf("P - Plot\n");
	printf("F - Filter stimulus\n");
	printf("I - multi-tone info\n");
	printf("Q - Quit\n");
	printf("? ");
	switch (getkey()) {
	case 'T':
	    filetype();
	    break;
	case 'V':
	    specval();
	    break;
	case 'C':
	    compute();
	    break;
	case 'M':
	    modify();
	    break;
	case 'W':
	    strstm();
	    break;
	case 'R':
	    (void) getstm();
	    break;
	case 'P':
	    plot();
	    break;
	case 'F':
	    filter();
	    break;
	case 'I':
	    printf("\n");
	    for (i = 0; i < ntone; i++)
		printf("%d  %d  %f  %f\n", i + 1, harm[i], amp[i], phase[i]);
	    break;
	case 'Q':
	    printf("\n");
	    exit(0);
	    break;
	}
    }
    return (0);
}

void
filetype()
{
    printf("\n\n");
    printf("FILE TYPE:\n");
    printf("\n");
    printf("1 - ILS\n");
    printf("2 - MAT\n");
    printf("? ");
    switch (getkey()) {
    case '1':
	filfmt = 0;
	break;
    case '2':
	filfmt = 1;
	break;
    }
}

void
compute()
{
    printf("\n\n");
    printf("COMPUTE STIMULUS:\n");
    printf("\n");
    printf("B - tone-burst\n");
    printf("C - single-tone\n");
    printf("D - compute impulse (delta)\n");
    printf("T - multi-tone\n");
    printf("I - compute low-pass impulse (click)\n");
    printf("L - log-sweep frequencies\n");
    printf("N - noise\n");
    printf("S - sweep frequencies\n");
    printf("Z - compute dispersion\n");
    printf("G - tone-gap\n");
    printf("? ");
    switch (getkey()) {
    case 'B':
	tobrst();
	break;
    case 'C':
	caltone();
	break;
    case 'D':
	delta();
	break;
    case 'T':
	multitone();
	break;
    case 'I':
	impulse();
	break;
    case 'L':
	logsweep();
	break;
    case 'N':
	noise();
	break;
    case 'S':
	sweep();
	break;
    case 'Z':
	dispersion();
	break;
    case 'G':
	tonegap();
	break;
    }
}

/*
 * scan and scale stimulus
 */
void
ss()
{
    int   i;
    double  ampmax, sc;

    ampmax = 1e-30;
    for (i = 0; i <= nper; i++)
	if (ampmax < astim[i])
	    ampmax = astim[i];
	else if (ampmax < -astim[i])
	    ampmax = -astim[i];
    sc = maxamp / ampmax;
    for (i = 0; i < nper; i++)
	astim[i] *= sc;
}

void
modify()
{
    printf("\n\n");
    printf("MODIFY STIMULUS:\n");
    printf("\n");
    printf("H - high-frequency emphasis\n");
    printf("L - low-frequency emphasis\n");
    printf("R - rotate\n");
    printf("S - scale maximum value\n");
    printf("W - window\n");
    printf("Z - zero\n");
    printf("? ");
    switch (getkey()) {
    case 'H':
	hfemph();
	break;
    case 'L':
	lfemph();
	break;
    case 'R':
	rotate();
	break;
    case 'S':
	ss();
	break;
    case 'W':
	winmod();
	break;
    case 'Z':
	zero();
	break;
    }
}

void
plot()
{
    printf("\n\n");
    printf("PLOT:\n");
    printf("\n");
    printf("F - frequency spectrum\n");
    printf("L - log-frequency spectrum\n");
    printf("T - time waveform\n");
    printf("? ");
    switch (getkey()) {
    case 'F':
	plotspec(fn, nper, delt, astim, 0);
	break;
    case 'L':
	plotspec(fn, nper, delt, astim, 1);
	break;
    case 'T':
	plotstm(fn, nper, delt, astim, maxamp);
	break;
    }
}

void
specval()
{
    long    n;
    double  rate;

    printf("\n");
    rate = getdouble("sampling rate (samples / sec) =  ", 1.0 / delt);
    if (rate > 0.0)
	delt = 1.0 / rate;
    n = getlong("stimulus length (0 to 8200 samples) =  ", (long) nper);
    if (n > 0)
	nper = n;
    n = getlong("maximum stimulus amplitude (0 to 32767) = ", (long) maxamp);
    if (n > 0 && n < 32768)
	maxamp = n;
}

void
getstr(char *s)
{
#ifdef linux
    fflush(stdout);
    kbd_restore();
    (void) fgets(s, STR_LEN, stdin);
    kbd_init();
#else
    (void) fgets(s, STR_LEN, stdin);
#endif
    s[strlen(s) - 1] = '\0';
}

double
getdouble(prompt, value)
char   *prompt;
double  value;
{
    char    str[STR_LEN];

    if (value > 1)
	printf("%s [%.3f] \t? ", prompt, value);
    else
	printf("%s [%.6f] \t? ", prompt, value);
    getstr(str);
    if (strlen(str) == 0) {
	return (value);
    } else {
	return (atof(str));
    }
}

long
getlong(prompt, value)
char   *prompt;
long    value;
{
    char    str[STR_LEN];
    long    atol();

    printf("%s [%ld] \t? ", prompt, value);
    getstr(str);
    if (strlen(str) == 0) {
	return (value);
    } else {
	return (atol(str));
    }
}

int
getyes(prompt, value)
char   *prompt;
int    value;
{
    int c;

    printf("%s (%s) \t? ", prompt, value ? "Y/n" : "y/N");
    for (;;) {
    	c = getkey();
    	if (c == 'y' || c == 'Y') {
            value = 1;
    	    break;
    	} else if (c == 'n' || c == 'N') {
            value = 0;
    	    break;
    	} else if (c == '\r' || c == '\n' || c == '\033') {
    	    break;
    	} else {
    	    beep();
    	}
    }
    return (value);
}

/*
 * zerstm - zero the stimulus
 */
void
zerstm()
{
    short   i;

    for (i = 0; i < MAXNPER; i++)
	astim[i] = 0.;
}

/****************************************************************************/

/*
 * hfemph
 */
void
hfemph()
{
    short   i;
    float   lstpnt;

    if (nper <= 0)
	return;
    lstpnt = astim[nper - 1];
    for (i = nper - 1; i >= 0; i--)
	astim[i] -= astim[i - 1];
    astim[0] -= lstpnt;
    ss();
}

/*
 * lfemph
 */
void
lfemph()
{
    int   i;

    if (nper <= 0)
	return;
    for (i = 1; i < nper; i++)
	astim[i] += astim[i - 1];
    ss();
}

void
rotate()
{
    int   i, ip, nofst = 0;
    float *bstim;

    printf("\n\n");
    nofst = getlong("number of samples to shift waveform = ", (long) nofst);
    if (nofst == 0)
	return;
    bstim = (float *) calloc(nper, sizeof(float));
    for (i = 0; i < nper; i++)
	bstim[i] = astim[i];
    ip = nper - nofst;
    for (i = 0; i < nper; i++)
	astim[i] = bstim[(i + ip) % nper];
    free(bstim);
}

void
zero()
{
    short     i, k, m;
    static double s = 0, d = 1;

    printf("\n");
    s = getdouble("start of zero (msec) = ", s);
    d = getdouble("duration of zero (msec) = ", d);
    if ((nper <= 0) || (delt <= 0.))
	return;
    d += s;
    if (s > 1000.0 * nper * delt)
    	s = 1000.0 * nper * delt;
    if (d > 1000.0 * nper * delt)
    	d = 1000.0 * nper * delt;

    k = (short) (0.001 * s / delt);
    m = (short) (0.001 * d / delt);
    for (i = k; i < m; i++)
	astim[i] = 0.;
}

void
beep()
{
    putchar('\007');
}

void
setlsb()
{
    short a = 1;
    char *b = (char *) (&a);

    cpu_lsb = b[1] ? 1 : 0;
}

int
getkey()
{
    int c;
    fflush(stdout);
    c = _getch();
    return (toupper(c));
}
