/* stmplt.c */

#include <stdio.h>
#include <math.h>
#include <string.h>
#define ANSI_C
#include <output.h>
#include <fftlib.h>
#include "stmgen.h"

#define BUF_LEN 10240
#define BELL     7
#define limit(min,val,max)	((val<min)?min:((val>max)?max:val))
#define _hypot   hypot

#ifdef linux
void    kbd_init();
void    kbd_restore();
#endif /* linux */

double  cdb();
double  cph();
int     getstim();
int     rdils();
void    gwindo();
void    tpln();

static void plot_spec();
static void plot_stm();
static void pltcy();
static void pltiy();
static void setlsb();
static void gtext();
static void usage();

extern short pc_video_adapt;
short   cpu_lsb = 0;

static double tpi = 6.2831853;
static float fftbuf[BUF_LEN] = {0};
static float xg = 1., yg = 1., xo = 0., yo = 0.;
static long buflen = BUF_LEN;
static short outbuf[BUF_LEN] = {0};
static FILE *ofp;
static int fn_flag = 0;
static int phase_type = 'D';

void
newext(f, e)
char   *f, *e;
{
    char   *s;

    s = f;
    while (*s != '\0')		/* locate end of filename */
	s++;
    while (*s != '\\' && *s != '/' && s > f)	/* backup to start of name */
	s--;
    while (*s != '\0' && *s != '.')	/* locate end of name */
	s++;
    while (*e != '\0')		/* copy extension */
	*s++ = *e++;
    *s = '\0';
}
int
main(argc, argv)
short   argc;
char  **argv;
{

    char   *ilstim, ofn[256];
    short   npts, maxamp;
    float   rate;
    char    plot_type = 'T';

    while (argc > 1 && argv[1][0] == '-') {
	switch (argv[1][1]) {
	case 'f':
	    plot_type = 'F';
	    break;
	case 'l':
	    plot_type = 'L';
	    break;
	case 'm':
	    phase_type = '\0';
	    break;
	case 'n':
	    fn_flag = 1;
	    break;
	case 'p':
	    phase_type = 'P';
	    break;
	default:
	    plot_type = 'T';
	}
	argc--;
	argv++;
    }
    if (argc < 2)
	usage();
    ilstim = argv[1];
    setlsb();

/*
 * read ils stimulus
 */
    maxamp = getstim(ilstim, outbuf, buflen, &rate, &npts);

/*
 * generate output file
 */
    if (fn_flag) {
	strcpy(ofn, ilstim);
	newext(ofn, ".plt");
    } else {
	strcpy(ofn, "stmplt.plt");
    }
    ofp = fopen(ofn, "w");
    fprintf(ofp, "; %s - plot of %s \n", ofn, ilstim);
    fprintf(ofp, ";\n");
    fprintf(ofp, "rotate=yes : wxlen=8 : wylen=10.5\n");
    fprintf(ofp, "pltype=lines : lintype=0\n");
    fprintf(ofp, "head=no : yhor=yes : clip=yes\n");
    fprintf(ofp, ";\n");
    switch (plot_type) {
    case 'F':
	plot_stm(ilstim, npts, 0.001 / rate, outbuf, maxamp);
	plot_spec(ilstim, npts, 0.001 / rate, outbuf, fftbuf, 0);
	break;
    case 'L':
	plot_stm(ilstim, npts, 0.001 / rate, outbuf, maxamp);
	plot_spec(ilstim, npts, 0.001 / rate, outbuf, fftbuf, 1);
	break;
    default:
	plot_stm(ilstim, npts, 0.001 / rate, outbuf, maxamp);
	plot_spec(ilstim, npts, 0.001 / rate, outbuf, fftbuf, 1);
    }
    fprintf(stderr, "output file is '%s'\n", ofn);

    return (0);
}

/*
 * getstim - Read ils stimulus file
 */

int
getstim(ilstim, obuf, nbuf, prate, pnpts)
char   *ilstim;
short  *obuf;
long    nbuf;
float  *prate;
short  *pnpts;
{
    int     i, mx;

    if (rdils(ilstim, obuf, nbuf, prate, pnpts) <= 0) {
	printf("can't open '%s'", ilstim);
	exit(1);
    }
    mx = 0;
    for (i = 0; i < *pnpts; i++)
	if (obuf[i] > 0 && mx < obuf[i])
	    mx = obuf[i];
	else if (obuf[i] < 0 && mx < -obuf[i])
	    mx = -obuf[i];
    return (mx);
}

void
setlsb()
{
    short   a = 1;
    char   *b = (char *) (&a);

    cpu_lsb = b[1] ? 1 : 0;
}

void
usage()
{
    fprintf(stderr, "usage: stmplt [-option] stim_file\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "option:\n");
    fprintf(stderr, "           -f  frequency spectrum\n");
    fprintf(stderr, "           -l  log-frequency spectrum\n");
    fprintf(stderr, "           -m  omit delay\n");
    fprintf(stderr, "           -n  use input file basename for output\n");
    exit(0);
}

/*******************************************************************/

void
plot_stm(ilstim, npts, delt, obuf, maxamp)
char   *ilstim;
short   npts, *obuf, maxamp;
float   delt;
{
/*
 * plot the waveform
 */
    char    str[80];
    char   *ts, *fs;
    short   i;
    double  tu, fu;
    double  y, ymn, ymx, ydc, sum, rms, dur;
    double  a;

    if (npts <= 0)
	return;
    a = maxamp;
    ymn = ymx = obuf[0];
    sum = 0.0;
    for (i = 0; i < npts; i++) {
	y = obuf[i];
	sum += y;
	if (ymx < y)
	    ymx = y;
	if (ymn > y)
	    ymn = y;
    }
    ydc = sum / npts;
    for (i = 0; i < npts; i++) {
	y = obuf[i] - ydc;
	sum += y * y;
    }
    if (npts * delt > 1) {
	tu = 1;
	ts = "sec";
    } else {
	tu = 1000;
	ts = "msec";
    }
    if (delt > 0.001) {
	fu = 1;
	fs = "Hz";
    } else {
	fu = 0.001;
	fs = "kHz";
    }
    dur = npts * delt * tu;
    rms = sqrt(sum / npts);
    fprintf(ofp, "\n");
    fprintf(ofp, "; waveform\n");
    fprintf(ofp, ";\n");
    fprintf(ofp, "newframe\n");
    fprintf(ofp, "xper=90 : xfmt=i : xmin=0 : xgain=%.3f\n", dur);
    fprintf(ofp, "yper=90 : yint=2.2\n");
    fprintf(ofp, "xlab=time (%s)\n", ts);
    fprintf(ofp, "ylab=\n");
    fprintf(ofp, "ymin=%.3f : ymax=%.3f\n", -a, a);
    if (dur < 10) {
	a = ceil(dur);
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=0\n", a, a + 0.2);
    } else if (dur < 20) {
	a = ceil(dur / 2) * 2;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=1\n", a, a);
    } else if (dur < 50) {
	a = ceil(dur / 5) * 5;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=0\n", a, a / 5 + 0.5);
    } else if (dur < 100) {
	a = ceil(dur / 10) * 10;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=0\n", a, a / 10 + 0.2);
    } else if (dur < 200) {
	a = ceil(dur / 20) * 20;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=1\n", a, a / 10);
    } else if (dur < 500) {
	a = ceil(dur / 50) * 50;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=0\n", a, a / 50 + 0.5);
    } else {
	a = ceil(dur / 100) * 100;
	fprintf(ofp, "xmax=%.3f : xint=%.1f : xans=0\n", a, a / 100 + 0.2);
    }
    gwindo(1.0, 8.0, 7.5, 10.0);/* define the 1st graphics window */
    pltiy(npts, obuf);		/* plot stimulus */

    sprintf(str, "file = %s", ilstim);
    gtext(0.0, -1.0, str);
    sprintf(str, "amplitude = %d", maxamp);
    gtext(4.0, -1.0, str);
    sprintf(str, "npts = %d", npts);
    gtext(0.0, -1.2, str);
    sprintf(str, "duration = %.3f %s", dur, ts);
    gtext(1.5, -1.2, str);
    if (delt > 0.0) {
	sprintf(str, "rate = %.1f %s", fu / delt, fs);
	gtext(4.0, -1.2, str);
    }
    sprintf(str, "min = %.0f    max = %.0f    dc = %.3f    rms = %.3f",
	ymn, ymx, ydc, rms);
    gtext(0.0, -1.4, str);
    fprintf(ofp, ";\n");
}

/* pltiy - plot integer array, y values only */

void
pltiy(n, iy)
short   n;
short  *iy;
{
    short   i;
    float   dx;

    fprintf(ofp, "data\n");
    dx = 1.0 / n;
    for (i = 0; i < n; i++) {
	fprintf(ofp, "%7.4f %8d\n", i * dx, iy[i]);
    }
    fprintf(ofp, "plot\n");
}

void
gwindo(x1, y1, x2, y2)
float   x1, y1, x2, y2;
{
    fprintf(ofp, "xllc=%.3f : xlen=%.3f\n", x1, x2 - x1);
    fprintf(ofp, "yllc=%.3f : ylen=%.3f\n", y1, y2 - y1);
}

void
tpln(x, y)
float  *x, *y;
{
    float   xx[2], yy[2];

    xx[0] = xo + xg * x[0];
    yy[0] = yo + yg * y[0];
    xx[1] = xo + xg * x[1];
    yy[1] = yo + yg * y[1];
    line(2, xx, yy);
}

/*******************************************************************/

void
plot_spec(ilstim, npts, delt, obuf, abuf, logx)
char   *ilstim;
short   npts;
float   delt;
short  *obuf;
float  *abuf;
short   logx;
{
    char   *ts, *fs;
    short   i, j, nfft, nfrq;
    double  xmn, xmx, xfa, a, dur;
    double  ymn, ymx, mxmg, phrf, mxgd;
    double  dt, df;
    double  tu, fu;
    float   mg, ph, gd;
    static char *fa[] = {
	"0.001", "0.002", "0.005", "0.01", "0.02", "0.05", "0.1", "0.2", "0.5",
        "1", "2", "5", "10", "20", "50", "100", "200", "500",
    };
    static int nfa = sizeof(fa) / sizeof(fa[0]);

    if (npts <= 0)
	return;
    for (nfft = 1; nfft < npts && nfft * 2 < MAXNPER; nfft *= 2)
	continue;
    if (npts * delt > 1) {
	tu = 1;
	ts = "sec";
    } else {
	tu = 1000;
	ts = "msec";
    }
    if (delt > 0.001) {
	fu = 1;
	fs = "Hz";
    } else {
	fu = 0.001;
	fs = "kHz";
    }
    nfrq = nfft / 2 + 1;
    for (i = 0; i < npts; i++)
	abuf[i] = obuf[i];
    for (i = npts; i < nfft; i++)
	abuf[i] = 0.0;
    fast(abuf, nfft);
    mxmg = 0.0;
    phrf = 0.0;
    dt = delt * nfft * tu;
    for (i = 0; i < nfrq; i++) {
	j = i * 2;
	mg = cdb(abuf[j], abuf[j + 1]);
	ph = cph(abuf[j], abuf[j + 1], phrf);
	gd = (i > 1) ? (ph - phrf) * dt : 0;
	if (mg > mxmg && i > 0)
	    mxmg = mg;
	abuf[j] = mg;
	abuf[j + 1] = gd;
	phrf = ph;
    }
    mxgd = 0.0;
    for (i = 0; i < nfrq; i++) {
	j = i * 2;
	mg = abuf[j];
	gd = abuf[j + 1];
	if (gd > mxgd && i > 0 && mg > (mxmg - 60))
	    mxgd = gd;
    }
    dur = mxgd;
    df = 0.5 * fu / delt;
    fprintf(ofp, "\n");
    fprintf(ofp, "; spectral magnitude\n");
    fprintf(ofp, ";\n");
    fprintf(ofp, "newframe\n");
    fprintf(ofp, "xper=90 : xfmt=i : xgain=%.3f : xcyc=%d \n", df, logx);
    fprintf(ofp, "yper=90 : yint=5.2\n");
    if (phase_type == '\0') {
        fprintf(ofp, "xlab= frequency\n");
    } else {
        fprintf(ofp, "xlab=\n");
    }
    fprintf(ofp, "ylab=magnitude (dB)\n");
    if (logx) {
	fprintf(ofp, "xann=");
	xmn = 2 * df / npts;
	xmx = df;
	for (i = 0; i < nfa; i++) {
	    xfa = atof(fa[i]);
	    if (xfa > xmn && xfa < xmx)
		fprintf(ofp, "%s ", fa[i]);
	}
	fprintf(ofp, "\n");
	fprintf(ofp, "xmin=%.3f : xmax=%.3f\n", xmn, xmx);
    } else {
	xmn = 0;
	if (df < 10) {
	    xmx = ceil(df);
	    fprintf(ofp, "xint=%.1f : xans=0 : ", xmx + 0.2);
	} else if (df < 20) {
	    xmx = ceil(df / 2) * 2;
	    fprintf(ofp, "xint=%.1f : xans=1 : ", xmx);
	} else if (df < 50) {
	    xmx = ceil(df / 5) * 5;
	    fprintf(ofp, "xint=%.1f : xans=0 : ", xmx / 5 + 0.5);
	} else if (df < 100) {
	    xmx = ceil(df / 10) * 10;
	    fprintf(ofp, "xint=%.1f : xans=0 : ", xmx / 10 + 0.2);
	} else if (df < 200) {
	    xmx = ceil(df / 20) * 20;
	    fprintf(ofp, "xint=%.1f : xans=1 : ", xmx / 10);
	} else if (df < 500) {
	    xmx = ceil(df / 50) * 50;
	    fprintf(ofp, "xint=%.1f : xans=1 : ", xmx / 50 + 0.5);
	} else {
	    xmx = ceil(df / 100) * 100;
	    fprintf(ofp, "xint=%.1f : xans=1 : ", xmx / 100 + 0.2);
	}
	fprintf(ofp, "xmin=%.3f : xmax=%.3f\n", xmn, xmx);
    }
    ymx = mxmg;
    ymn = ymx - 100.0;
    fprintf(ofp, "ymin=%.3f : ymax=%.3f\n", ymn, ymx);
    if (phase_type == '\0') {
        gwindo(1.0, 1.5, 7.5, 6.0);
    } else {
        gwindo(1.0, 4.0, 7.5, 6.0);
    }
    pltcy(nfrq, abuf, logx);	/* plot log magnitude */

    if (phase_type == 'D') {
        fprintf(ofp, "\n");
        fprintf(ofp, "; group delay\n");
        fprintf(ofp, ";\n");
        fprintf(ofp, "newframe\n");
        fprintf(ofp, "xlab=frequency (%s)\n", fs);
        fprintf(ofp, "ylab=group delay (%s)\n", ts);
        fprintf(ofp, "xmin=%.3f : xmax=%.3f\n", xmn, xmx);
        fprintf(ofp, "ymin=0 : ");
        if (dur < 5) {
            a = ceil(dur);
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=0\n", a, a + 0.2);
        } else if (dur < 10) {
            a = ceil(dur / 2) * 2;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=1\n", a, a);
        } else if (dur < 20) {
            a = ceil(dur / 5) * 5;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=0\n", a, a / 5 + 0.5);
        } else if (dur < 50) {
            a = ceil(dur / 10) * 10;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=0\n", a, a / 10 + 0.2);
        } else if (dur < 100) {
            a = ceil(dur / 20) * 20;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=1\n", a, a / 10);
        } else if (dur < 200) {
            a = ceil(dur / 50) * 50;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=0\n", a, a / 50 + 0.5);
        } else if (dur < 500) {
            a = ceil(dur / 100) * 100;
            fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=0\n", a, a / 100 + 0.2);
        } else {
            a = ceil(dur / 200) * 200;
    	    fprintf(ofp, "ymax=%.3f : yint=%.1f : yans=1\n", a, a / 100);
        }
        gwindo(1.0, 1.5, 7.5, 3.5);	/* define the 1st graphics window */
        pltcy(nfrq, abuf + 1, logx);/* plot group delay */
    }
}

/* pltcy - plot complex array */
void
pltcy(n, cy, logx)
short   n;
float   cy[];
{
    short   i;
    double  dx;

    fprintf(ofp, "data\n");
    dx = 1.0 / n;
    for (i = 1; i < n; i++) {
	fprintf(ofp, "%8.4f %8.2f\n", i * dx, cy[i * 2]);
    }
    fprintf(ofp, "plot\n");
}

double
cdb(x, y)
double  x, y;
{
    double  mg, db;

    mg = _hypot(x, y);
    db = (mg < 1.e-39) ? -800.0 : 20.0 * log10(mg);
    return (db);
}

double
cph(x, y, r)
double  x, y, r;
{
    double  ph;

    if (x == 0.0 && y == 0.0)
	return (0.0);
    ph = atan2(x, y) / tpi;
    while (ph - r > 0.5)
	ph -= 1.0;
    while (ph - r < -0.5)
	ph += 1.0;
    while (ph - r < 0)
	ph += 1.0;
    return (ph);
}

void
getstr(char *s)
{
#ifdef linux
    fflush(stdout);
    kbd_restore();
    (void) fgets(s, STR_LEN, stdin);
    kbd_init();
#else
    (void) fgets(s, STR_LEN, stdin);
#endif
    s[strlen(s) - 1] = '\0';
}

void
gtext(x, y, s)
double  x, y;
char   *s;
{
    fprintf(ofp, "%.2f %.2f \"%s\"\n", x, y, s);
}
