/* strstm - store stimulus to disk */

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "stmgen.h"
#ifdef MSDOS
#include <io.h>
#else
#define _O_BINARY    0
#define _O_CREAT     O_CREAT
#define _O_RDWR      O_RDWR
#define _close       close
#define _lseek       lseek
#define _write       write
#define _read        read
#endif
#ifdef linux
#include <unistd.h>
void	swab();
#endif
#define PMODE        0644
#define _open        open
#define _swab        swab

#define OFLAG       ((int) (_O_RDWR | _O_BINARY | _O_CREAT))
#define wswapb(a,b) _swab((char *)(a), (char *)(a), (int) ((b)*sizeof(short)))
#define nint(x)     ((int)(((x) < 0)?((x)-0.5):((x)+0.5)))


char   *strchr();
unsigned int strlen();
void   getstr();

extern char fn[STR_LEN];
extern float delt, f0, amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short cpu_lsb;
extern short nper, nblk, maxamp, harm[MAXNTONE];
extern short params[256];
extern short ilshdr[64];
extern short istim[MAXNPER];
extern short filfmt;

static void wrils();
static void wrmat();

static short ichan = 0;
static short nchan = 1;

void
strstm()
{
    static char *ffn[2] = {"ILS", "MAT"};

    printf("\nWrite stimulus to file (%s): ", ffn[filfmt]);
    getstr(fn);
    if (strlen(fn) <= 0)
	return;

    switch (filfmt) {
    case 0:
        wrils();
	break;
    case 1:
        wrmat();
	break;
    }
}

void
wrils()
{
    float     r;
    short     i, rate, bytes, ipwr, fd;

    for (i = 0; i < nper; i++)
        istim[i] = nint(astim[i]);

    nblk = 1 + (nper - 1) / 256;
    r = 1 / delt;
    for (ipwr = 0; r > 32767; ipwr++)
        r /= 10;
    rate = (short) r;

    /* make header */
    params[5] = nblk;
    params[57] = ichan;
    params[58] = nchan;
    params[60] = ipwr;
    params[61] = rate;
    params[62] = ILS;
    fd = _open(fn, OFLAG, PMODE);/* open the file */
    if (fd == -1)
	perror("open failed");
    if (cpu_lsb != 0)
	wswapb(params, BLKSIZ);
    bytes = _write(fd, (char *) params, BLKSIZ * sizeof(short));
    if (bytes == -1) {
	perror("write error");
    }
    if (cpu_lsb != 0)
	wswapb(istim, nper);
    _write(fd, (char *) istim, nper * sizeof(short));
    _close(fd);
}

/*
 *  matlab header -
 *	type = MOPT where:
 *	M=0 for pc; 1 for sun
 *	O=0 col or 1 row (ignored);
 *	P= [0,1,2,3,4]=>[r*8,r*4,I*4,I*2,I*4 unsigned integers]
 *	T= 0 for matrix or 1 for text (stored as r*4 numbers 0<i<255).
 */
static long
encode_mopt(char *type)
{
    return(type[0] * 1000 + type[2] * 10 + type[3]);
}

void
wrmatf(int fd, char *nam, float *s, int n)
{
    char    type[4];
    long    ns, hdr[5];

    ns = strlen(nam) + 1;
    type[0] = cpu_lsb;		/* byte swap = 0 for PC, 1 for SUN */
    type[1] = 0;		/* ignored */
    type[2] = 1;		/* R*4 */
    type[3] = 0;		/* 0 for matrix */
    hdr[0] = encode_mopt(type); /* MOPT */
    hdr[1] = n; 		/* rows */
    hdr[2] = 1;			/* cols */
    hdr[3] = 0;			/* 0 for real, 1 for complex */
    hdr[4] = ns;	        /* name length (including null byte) */
    (void) _write(fd, hdr, 20);
    (void) _write(fd, nam, (int) ns);
    _write(fd, s, n * sizeof(float));
}

void
wrmat()
{
    float   rate;
    int     fd;

    fd = _open(fn, OFLAG, PMODE);    /* open the file */
    if (fd == -1) {
	perror("open failed");
        return;
    }
    rate = 1 / delt;
    wrmatf(fd, "rate", &rate, 1); 
    wrmatf(fd, "stim", astim, nper);
    _close(fd);
}
