/*
 * sweep - sweep frequencies
 */
#include <stdio.h>
#include <math.h>
#include <fftlib.h>
#include "stmgen.h"

#define MAXNPOLE    64

void    ss();
void    zerstm();

extern float freq;
extern float dur;
extern float delt;
extern float amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short maxamp;
extern short nper;
extern short harm[MAXNTONE];

double  getdouble();
long    getlong();
struct cmplx {
    float   x, y;
};

static double tpi = (2.0 * 3.1415927);

void
sweep()
{
    short   i;
    double  angle, dw, w;
    static double swrt = 0, a = 1;

    printf("\n");
    a = getdouble("sweep rate multiplier = ", a);
/*
 * define stimulus parameters
 */
    harm[0] = 0;
    amp[0] = 1.0;
    phase[0] = 0.0;

/*
 * compute stimulus
 */
    zerstm();

/*
 * compute frequency response
 */
    swrt = tpi * nper * a;
    dw = 1.0 / nper;
    for (i = 0; i <= (nper / 2); i++) {
	w = i * dw;
	angle = -swrt * w * w;
	astim[2 * i] = cos(angle);
	astim[2 * i + 1] = sin(angle);
    }
    astim[0] = 0.0;
    astim[1] = 0.0;
    astim[nper] = 0.0;
    astim[nper + 1] = 0.0;

/*
 * do fourier synthesis
 */
    fsst(astim, nper);

/*
 * scale stimlulus
 */
    ss();
}

void
logsweep()
{
    short   i;
    double  df, c, f, h, p;
    static double b = 0, fl = 0, fh = 0;

    printf("\n");
    fl = getdouble("low frequency (Hz) = ", fl);
    fh = getdouble("high frequency (Hz) = ", fh);
    if (b == -1)
	return;

/*
 * define stimulus parameters
 */
    harm[0] = 0;
    amp[0] = 1.0;
    phase[0] = 0.0;

/*
 * compute stimulus
 */
    zerstm();

/*
 * compute frequency response
 */
    df = 1 / (delt * nper);	/* delta frequency */
    c = log(nper / 2.0) / (df * df);
    astim[0] = 0.0;
    astim[1] = 0.0;
    for (i = 1; i <= (nper / 2); i++) {
	f = i * df;
        h = 1;
	if (fh > 0)
            h *= 1 / (1 + pow(f / fh, 2));
	if (fl > 0)
            h *= 1 / (1 + pow(fl / f, 2));
	p = -tpi * f * c * (log(f / df) - 1);
	astim[2 * i] = h * cos(p);
	astim[2 * i + 1] = h * sin(p);
    }
    astim[nper] = 0.0;
    astim[nper + 1] = 0.0;

/*
 * do fourier synthesis
 */
    fsst(astim, nper);

/*
 * scale stimlulus
 */
    ss();
}

void
dispersion()
{
    short   i;
    double  df, c, f, h, p;
    static double a = -5, b = -0.5, fl = 0, fh = 0, f1 = 1000;

    printf("\n");
    a = getdouble("1 kHz delay (ms) = ", a);
    b = getdouble("slope of log-time vs log-frequency = ", b);
    fl = getdouble("low frequency (Hz) = ", fl);
    fh = getdouble("high frequency (Hz) = ", fh);
    if (b == -1)
	return;

/*
 * define stimulus parameters
 */
    harm[0] = 0;
    amp[0] = 1.0;
    phase[0] = 0.0;

/*
 * compute stimulus
 */
    zerstm();

/*
 * compute frequency response
 */
    df = 1 / (delt * nper);	/* delta frequency */
    c = -0.001 * a * pow(f1, -b) / (b + 1) * (40 / tpi);
    astim[0] = 0.0;
    astim[1] = 0.0;
    for (i = 1; i <= (nper / 2); i++) {
	f = i * df;
        h = 1;
	if (fh > 0)
            h *= 1 / (1 + pow(f / fh, 2));
	if (fl > 0)
            h *= 1 / (1 + pow(fl / f, 2));
	p = c * pow(f, b + 1);
	astim[2 * i] = h * cos(p);
	astim[2 * i + 1] = h * sin(p);
    }
    astim[nper] = 0.0;
    astim[nper + 1] = 0.0;

/*
 * do fourier synthesis
 */
    fsst(astim, nper);

/*
 * scale stimlulus
 */
    ss();
}
