/*
 * tone - Compute tone stimulus
 */
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "stmgen.h"

void    ss();

extern float freq;
extern float dur;
extern float delt;
extern float amp[MAXNTONE], phase[MAXNTONE];
extern float astim[MAXNPER];
extern short maxamp;
extern short nper;
extern short ntone;
extern short harm[MAXNTONE];

int     getyes();
long    getlong();
double  getdouble();

static void tonpip();
static void window();

static double gsd = 1;
static double tpi = 6.2831853;
static short nwin = 2;
static short rmdc = 0;

double
vsum(float *v, short n)
{
    int i;
    double sum = 0;
    
    for (i = 0; i < n; i++)
        sum += v[i];
    return (sum);
}

void
tobrst()
{
    int     i;
    double  h, deg = 90.0, ofst1, ofst2, bl = 0;

    printf("\n");
    printf("window type (0=Hanning, 1=Hamming, 2=Blackman, 3=Nuttall1 \n");
    nwin = getlong("    4=Nuttall2, 5=Bartlet, 6=Gaussian, 7=Rectangular) = ",
	(long) nwin);
    if (nwin == 6)
        gsd = getdouble("Gaussian standard deviation = ", gsd);
    freq = getdouble("frequency of tone (Hz) = ", freq);
    dur = getdouble("duration of burst (sec) = ", dur);
    deg = getdouble("sine phase at center (deg) = ", deg);
    rmdc = getyes("remove dc", rmdc);
    if ((nper <= 0) || (delt <= 0.))
	return;
    if (dur > MAXNPER * delt)
    	dur = MAXNPER * delt;
    h = freq * delt * nper;

/*
 * define stimulus parameters
 */
    ntone = 1;
    harm[0] = (short) h;
    amp[0] = 1.;
    phase[0] = 90.;

/*
 * compute stimulus
 */
    if (rmdc) {
        tonpip(astim, nper, delt, 0.0, dur, deg, nwin, bl);
        ofst1 = vsum(astim, nper);
    }
    tonpip(astim, nper, delt, freq, dur, deg, nwin, bl);
    if (rmdc && ofst1 > 0) {
        ofst2 = vsum(astim, nper);
        for (i = 0; (i < 8) && (ofst2 > 1e-5); i++) {
            bl -= ofst2 / ofst1;
            tonpip(astim, nper, delt, freq, dur, deg, nwin, bl);
            ofst2 = vsum(astim, nper);
        }
    }
    for (i = 0; i < nper; i++)
        astim[i] *= maxamp;
}

void
caltone()
{
    short     i;
    float   a, h;

    printf("\n");
    freq = getdouble("frequency of tone (Hz) = ", freq);
    if ((nper <= 0) || (delt <= 0.))
	return;
    h = floor(freq * delt * nper + 0.5);
    freq = h / (delt * nper);
    printf("tone frequency = %9.1f Hz\n", freq);
    printf("number of cycles per sweep = %9.1f\n", h);
/*
 * define stimulus parameters
 */
    ntone = 1;
    harm[0] = (short) h;
    amp[0] = 1.;
    phase[0] = 90.;

/*
 * compute stimulus
 */
    a = tpi * h / nper;
    for (i = 0; i < nper; i++)
	astim[i] = maxamp * sin(i * a);
}

void
multitone()
{
    short     i, j;
    double    g, p, s;
    float     a, h;
    static double degrad = 3.14165927 / 180.0;

    printf("\n");
    if ((nper <= 0) || (delt <= 0.))
	return;
/*
 * define stimulus parameters
 */
    for (i = 0; i < MAXNTONE; i++) {
	printf("\ntone #%d\n", i + 1);
        freq = getdouble("frequency (Hz) = ", 0.0);
        h = floor(freq * delt * nper + 0.5);
	if (h <= 0)
	    break;
        freq = h / (delt * nper);
	g = getdouble("relative amplitude =", 1.0);
	if (g < 0)
	    g = pow(10.0, g / 20);
	p = getdouble("sine phase (degrees) =", 0.0);
        printf("\n");
	printf("tone #%d frequency = %9.1f Hz\n", i + 1, freq);
	printf("tone #%d cycles    = %.0f / sweep\n", i + 1, h);
	printf("tone #%d amplitude = %9.1f\n", i + 1, g);
	printf("tone #%d phase     = %9.1f deg\n", i + 1, p);
	harm[i] = (short) h;
	amp[i] = g;
	phase[i] = p;
    }
    ntone = i;

/*
 * compute stimulus
 */
    for (i = 0; i < nper; i++) {
	s = 0;
	for (j = 0; j < ntone; j++) {
	    a = i * tpi * harm[j] / nper + phase[j] * degrad;
	    s += amp[j] * sin(a);
	}
	astim[i] = s * maxamp;
    }

/*
 * scan and scale
 */
    ss();
}

/*
 * tonpip
 */
void
tonpip(w, n, dt, fr, bd, deg, wt, bl)
float   *w;
short     n, wt;
double   dt, fr, bd, deg, bl;
{
    double   ph, dp, temp, mc, phc;
    short     i, m;

    m = (short) (bd / dt);
    phc = deg * tpi / 360.0;
    if (fr <= 0) {
	for (i = 0; i < m; i++)
	    w[i] = 1 + bl;
    } else {
	dp = tpi * fr * dt;
	mc = (m - 1) / 2.0;
	for (i = 0; i < m; i++) {
	    ph = (i - mc) * dp;
	    temp = sin(ph + phc);
	    w[i] = temp + bl;
	}
    }
    window(w, m, wt);
    for (i = m; i < MAXNPER; i++)
	w[i] = 0.;
}
/*
 * window -
 *
 *	computes window function given parameters as described by nutall
 *	(1981) "some windows with very good sidelobe behavior,"
 *	ieee transactions on acoustics, speech, and signal processing,
 *	assp-29, 84-91.
 *
 *	usage:
 *
 *	call window (wave,m,wt)
 *
 *	where:
 *		wave is the waveform array to be windowed.
 *		wt is the window type: 
 *		0=Hanning, 1=Hamming, 2=Blackman, 3=Nuttall1
 *              4=Nuttall2,5=Bartlet, 6=Gaussian, 7=Rectangular
 */
void
window(wave, m, wt)
float   *wave;
short     m, wt;
{
    float   dp, ph, w, mc, s;
    short     i, k, c;
    static float Hanning[3] = {0.5, 0.5};
    static float Hamming[3] = {0.53836, 0.46164};
    static float Blackman[3] = {0.42, 0.5, 0.08};
    static float Nuttall1[3] = {0.40897, 0.5, 0.09103};
    static float Nuttall2[3] = {0.4243801, 0.4973406, 0.0782793};
    static float *a[] = {Hanning, Hamming, Blackman, Nuttall1, Nuttall2};
    static short ncoef[] = {2, 2, 3, 3, 3};

    mc = m / 2.0;
    if (wt >= 0 && wt <= 4) {
        dp = tpi / m;
        for (i = 0; i < m; i++) {
    	    ph = (i - mc) * dp;
    	    w = a[wt][0];
    	    for (k = 1; k < ncoef[wt]; k++)
    	        w = w + a[wt][k] * cos(ph * k);
    	    wave[i] = wave[i] * w;
        }
    } else if (wt == 5) {	/* Bartlet */
	c = (m + 1) / 2;
        for (i = 0; i < c; i++) {
    	    wave[i] = wave[i] * (i / mc);
	}
        for (i = c; i < m; i++) {
    	    wave[i] = wave[i] * (2 - i / mc);
	}
    } else if (wt == 6) {	/* Gaussian */
	s = gsd / sqrt(2.0) / mc;
        for (i = 0; i < m; i++) {
    	    w = s * (i - mc);
    	    wave[i] = wave[i] * exp(-w * w);
	}
    } else if (wt == 7) {	/* Rectangular */
    }
}

void
winmod()
{
    short     i, k, m;
    static double s = 0, d = 1;

    printf("\n");
    printf("window type (0=Hanning, 1=Hamming, 2=Blackman, 3=Nuttall1 \n");
    nwin = getlong("    4=Nuttall2, 5=Bartlet, 6=Gaussian, 7=Rectangular) = ",
	(long) nwin);
    d = getdouble("duration of window (msec) = ", d);
    s = getdouble("start of window (msec) = ", s);
    if ((nper <= 0) || (delt <= 0.))
	return;
    if (d > 1000.0 * nper * delt)
    	d = 1000.0 * nper * delt;
    if (s > 1000.0 * nper * delt)
    	s = 1000.0 * nper * delt;

    m = (short) (0.001 * d / delt);
    k = (short) (0.001 * s / delt);
    for (i = 0; i < k; i++)
	astim[i] = 0.;
    window(astim + k, m, nwin);
    for (i = m + k; i < nper; i++)
	astim[i] = 0.;
}

void
tonegap()
{
    short     i, j;
    double    g, f, a;
    static double tonfrq = 0;
    static double gapsiz = 0;
    static short gapint = 0;

    printf("\n");
    if ((nper <= 0) || (delt <= 0.))
	return;
/*
 * define stimulus parameters
 */
    tonfrq = getdouble("frequency (Hz) = ", tonfrq);
    f = tonfrq * delt;
    gapint = getlong("gap interval (points) = ", (long) gapint);
    gapsiz = getdouble("gap size (points) = ", gapsiz);
    harm[0] = (short) floor(f * nper + 0.5);
    amp[0] = 1;
    phase[0] = 0;
    ntone = 1;

/*
 * compute stimulus
 */
    if (gapint <= 0 || gapsiz <= 0) {
        for (i = 0; i < nper; i++) {
            a = i * tpi * f;
            astim[i] = maxamp * sin(a);
        }
    } else {
        for (i = 0; i < nper; i++) {
            j = i / gapint;
            g = i + j * gapsiz;
            a = g * tpi * f;
            astim[i] = maxamp * sin(a);
        }
    }
}

